#=========================================================================================>
# Author: Emmanuel Selorm Tsyawo
# Project: Feasible IV Regression without Excluded Instruments
# Date began:   Sept 6, 2021
# Last Update:  March 7, 2021
# Place:        Rabat
#=========================================================================================>
# Load packages, clear memory,
#rm(list=ls())
library(estrpac)#install from binary file estrpac_0.1.0.tgz or
# use devtools::install_github("estsyawo/estrpac")
library(pbapply)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) #set working directory path to source file
list.files()
#=========================================================================================>
# Comments: Monte Carlo Simulations using the DGP of Escanciano 2018 (Online Appendix)
# n = 250; DGP_3A and DGP_3B
#=========================================================================================>
# Set Parameters
R = 1000 # set number of simulations
b=1#parameter of interest

RMSE = function(x) sqrt(mean(x^2))
MAD = function(x) median(abs(x))

#-------------------------------------------------------

gdat1<- function(n,f.fun,gam){
  datl=list()
  for (l in 1:R) {
    set.seed(l) #generate seed
    Z=rnorm(n)
    Sig = diag(2); Sig[1,2]=Sig[2,1]=0.8
    set.seed(l+10)
    EU=MASS::mvrnorm(n=n,mu=c(0,0),Sigma = Sig)
    #generate endogenous covariate
    D = f.fun(Z,EU[,2],gam)
    #generate outcome
    Y = 1 + b*D + EU[,1]
    datl[[l]]<- data.frame(Y,D,Z)
  }
  datl
}
#transformations of Z for the endogenous variable
f.fun1=function(Z,U,gam){gam*Z+U}
f.fun2=function(Z,U,gam){1*((1+gam*Z+U)>0)}

# size a, n = 250
# gamma=0.1
datl1a=gdat1(n=250,f.fun=f.fun1,gam = 0.1) #linear homoskedastic model

summary(imlmreg2.fit(Y=datl1a[[1]]$Y,X=datl1a[[1]]$D,Z=datl1a[[1]][,-c(1,2)]))#MMD is the default
summary(imlmreg2.fit(Y=datl1a[[1]]$Y,X=datl1a[[1]]$D,Z=datl1a[[1]][,-c(1,2)],Kern = "Gauss.W"))
summary(imlmreg2.fit(Y=datl1a[[1]]$Y,X=datl1a[[1]]$D,Z=datl1a[[1]][,-c(1,2)],Kern = "Gauss"))
summary(imlmreg2.fit(Y=datl1a[[1]]$Y,X=datl1a[[1]]$D,Z=datl1a[[1]][,-c(1,2)],Kern = "DL"))
summary(imlmreg2.fit(Y=datl1a[[1]]$Y,X=datl1a[[1]]$D,Z=datl1a[[1]][,-c(1,2)],Kern = "WMD"))

MCfun1<- function(j=NULL,datl,b){
  dat = datl[[j]] #null operation for looping
  Y = dat[,1]; X = as.matrix(dat[,2])
  Z1 = as.matrix(dat[,3]); Z2 = as.matrix(dat[,-c(1,2)])
  
  #run Integrated Moment regressions
  MMDobj=imlmreg2.fit(Y,X,Z1,Kern = "Euclid")
  WMDobj=imlmreg2.fit(Y,X,Z1,Kern = "WMD")
  WMDFobj=imlmreg2.fit(Y,X,Z1,Kern = "WMDF")
  DLobj=imlmreg2.fit(Y,X,Z1,Kern = "DL")
  Esc6obj=imlmreg2.fit(Y,X,Z1,Kern = "Esc6")
  IIVobj=imlmreg2.fit(Y,X,Z1,Kern = "Gauss.W")
  #run K-Class regressions
  TSLSobj=ivreg(Y~X|Z2); TSLSobj$HC_Std.Err=sqrt(diag(vcovHC(TSLSobj)))
  JIVEobj=kClassIVreg.fit(Y,X,Z2,method = "JIVE")
  LIMLobj=kClassIVreg.fit(Y,X,Z2,method = "LIML")
  HLIMobj=kClassIVreg.fit(Y,X,Z2,method = "HLIM")
  HFULobj=kClassIVreg.fit(Y,X,Z2,method = "HFUL")
  
  #compute bias
  bias.MMD = MMDobj$coefficients[2]-b 
  bias.WMD = WMDobj$coefficients[2]-b
  bias.WMDF = WMDFobj$coefficients[2]-b
  bias.DL = DLobj$coefficients[2]-b
  bias.Esc6 = Esc6obj$coefficients[2]-b
  bias.IIV = IIVobj$coefficients[2]-b
  
  bias.TSLS = TSLSobj$coefficients[2]-b
  bias.JIVE = JIVEobj$coefficients[2]-b
  bias.LIML = LIMLobj$coefficients[2]-b
  bias.HLIM = HLIMobj$coefficients[2]-b
  bias.HFUL = HFULobj$coefficients[2]-b
  
  #compute size (for rejection probabilities)
  size.MMD = 1*(abs(bias.MMD/MMDobj$HC_Std.Err[2])>qnorm(0.975))
  size.WMD = 1*(abs(bias.WMD/WMDobj$HC_Std.Err[2])>qnorm(0.975))
  size.WMDF = 1*(abs(bias.WMDF/WMDFobj$HC_Std.Err[2])>qnorm(0.975))
  size.DL = 1*(abs(bias.DL/DLobj$HC_Std.Err[2])>qnorm(0.975))
  size.Esc6 = 1*(abs(bias.Esc6/Esc6obj$HC_Std.Err[2])>qnorm(0.975))
  size.IIV = 1*(abs(bias.IIV/IIVobj$HC_Std.Err[2])>qnorm(0.975))
  
  size.TSLS = 1*(abs(bias.TSLS/TSLSobj$HC_Std.Err[2])>qnorm(0.975))
  size.JIVE = 1*(abs(bias.JIVE/JIVEobj$HC_Std.Err[2])>qnorm(0.975))
  size.LIML = 1*(abs(bias.LIML/LIMLobj$HC_Std.Err[2])>qnorm(0.975))
  size.HLIM = 1*(abs(bias.HLIM/HLIMobj$HC_Std.Err[2])>qnorm(0.975))
  size.HFUL = 1*(abs(bias.HFUL/HFULobj$HC_Std.Err[2])>qnorm(0.975))
  
  res=c(bias.MMD,bias.WMD,bias.WMDF,bias.DL,bias.Esc6,bias.IIV,bias.TSLS,bias.JIVE,bias.LIML,bias.HLIM,
        bias.HFUL,size.MMD,
        size.WMD,size.WMDF,size.DL,size.Esc6,size.IIV,size.TSLS,size.JIVE,size.LIML,
        size.HLIM,size.HFUL)
  names(res)=c("BiasMMD","BiasWMD","BiasWMDF","BiasDL","BiasEsc6","BiasIIV","Bias.TSLS",
               "Bias.JIVE","Bias.LIML","Bias.HLIM","Bias.HFUL",
               "SizeMMD","SizeWMD","SizeWMDF","SizeDL","SizeEsc6","SizeIIV","Size.TSLS",
               "Size.JIVE","Size.LIML","Size.HLIM","Size.HFUL")
  res
}

# 
#-------------------------------------------------------

# illustration
MCfun1(j=2,datl=datl1a,b=b)

#linear homoskedastic model, n = 250, gamma = 0.1
Bias1a = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1a,b=b)
summary(t(Bias1a))

# gamma = 0.25
datl1aM=gdat1(n=250,f.fun=f.fun1,gam = 0.25)
Bias1a.M = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1aM,b=b)
summary(t(Bias1a.M))

# gamma = 0.5
datl1aMM=gdat1(n=250,f.fun=f.fun1,gam = 0.5) #linear heteroskedastic model
Bias1a.MM = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1aMM,b=b)
summary(t(Bias1a.MM))
#-------------------------------------------------------
# size b, n = 250

#non-linear (binary) homoskedastic model, n = 250, gamma = 0.1
datl1b=gdat1(n=250,f.fun=f.fun2,gam = 0.1)
Bias1b = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1b,b=b)
summary(t(Bias1b))

# gamma = 0.5
datl1bM=gdat1(n=250,f.fun=f.fun2,gam = 0.5)
Bias1b.M = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1bM,b=b)
summary(t(Bias1b.M))

# gamma = 1
datl1bMM=gdat1(n=250,f.fun=f.fun2,gam = 1)
Bias1b.MM = pbsapply(1:R, FUN=MCfun1,cl=5,datl=datl1bMM,b=b)
summary(t(Bias1b.MM))
#=========================================================================================>

#=========================================================================================>

# Results:

Res.A1=round(cbind(apply(Bias1a[1:11,],1,mean),apply(Bias1a[1:11,],1,MAD),apply(Bias1a[1:11,],1,RMSE),
                   apply(Bias1a[12:22,],1,mean)),3)
colnames(Res.A1)=c("MB","MAD","RMSE","Rej")
rownames(Res.A1)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.A1

Res.A2=round(cbind(apply(Bias1a.M[1:11,],1,mean),apply(Bias1a.M[1:11,],1,MAD),apply(Bias1a.M[1:11,],1,RMSE),
                   apply(Bias1a.M[12:22,],1,mean,na.rm=TRUE)),3)
colnames(Res.A2)=c("MB","MAD","RMSE","Rej")
rownames(Res.A2)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.A2

Res.A3=round(cbind(apply(Bias1a.MM[1:11,],1,mean),apply(Bias1a.MM[1:11,],1,MAD),apply(Bias1a.MM[1:11,],1,RMSE),
                   apply(Bias1a.MM[12:22,],1,mean)),3)
colnames(Res.A3)=c("MB","MAD","RMSE","Rej")
rownames(Res.A3)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.A3

cbind(Res.A1,Res.A2,Res.A3)

#         MB   MAD   RMSE   Rej     MB   MAD  RMSE   Rej     MB   MAD  RMSE   Rej
# MMD   0.252 0.360  0.802 0.204  0.004 0.155 0.270 0.082  0.001 0.081 0.126 0.053
# WMD   0.424 0.424  7.439 0.105 -0.113 0.177 1.513 0.067 -0.011 0.090 0.142 0.053
# WMDF  0.107 0.408  6.889 0.111 -0.065 0.176 0.480 0.068 -0.010 0.089 0.141 0.055
# DL    0.026 0.426 28.890 0.182  0.037 0.162 0.896 0.072 -0.001 0.084 0.130 0.054
# ESC6  0.252 0.347  0.618 0.179 -0.007 0.163 0.271 0.073 -0.002 0.084 0.130 0.055
# IIV   0.190 0.404  2.143 0.195  0.003 0.173 0.286 0.081  0.000 0.088 0.136 0.060
# TSLS -0.255 0.401  5.934 0.094 -0.058 0.161 0.315 0.060 -0.012 0.081 0.127 0.047
# JIVE  1.633 0.602 48.693 0.160 -0.275 0.185 3.012 0.045 -0.026 0.083 0.135 0.039
# LIML -0.255 0.401  5.934 0.094 -0.058 0.161 0.315 0.060 -0.012 0.081 0.127 0.047
# HLIM  1.633 0.602 48.693 0.160 -0.275 0.185 3.012 0.045 -0.026 0.083 0.135 0.039
# HFUL  0.060 0.394  5.869 0.102 -0.056 0.160 0.315 0.061 -0.011 0.080 0.127 0.048

#for pasting into TeX
apply(cbind(Res.A1,Res.A2,Res.A3),1,paste,collapse=" & ")

# MMD 0.252 & 0.36 & 0.802 & 0.204 & 0.004 & 0.155 & 0.27 & 0.082 & 0.001 & 0.081 & 0.126 & 0.053" 
# WMD 0.424 & 0.424 & 7.439 & 0.105 & -0.113 & 0.177 & 1.513 & 0.067 & -0.011 & 0.09 & 0.142 & 0.053" 
# WMDF 0.107 & 0.408 & 6.889 & 0.111 & -0.065 & 0.176 & 0.48 & 0.068 & -0.01 & 0.089 & 0.141 & 0.055" 
# DL 0.026 & 0.426 & 28.89 & 0.182 & 0.037 & 0.162 & 0.896 & 0.072 & -0.001 & 0.084 & 0.13 & 0.054" 
# ESC6 0.252 & 0.347 & 0.618 & 0.179 & -0.007 & 0.163 & 0.271 & 0.073 & -0.002 & 0.084 & 0.13 & 0.055" 
# IIV 0.19 & 0.404 & 2.143 & 0.195 & 0.003 & 0.173 & 0.286 & 0.081 & 0 & 0.088 & 0.136 & 0.06" 
# TSLS -0.255 & 0.401 & 5.934 & 0.094 & -0.058 & 0.161 & 0.315 & 0.06 & -0.012 & 0.081 & 0.127 & 0.047" 
# JIVE 1.633 & 0.602 & 48.693 & 0.16 & -0.275 & 0.185 & 3.012 & 0.045 & -0.026 & 0.083 & 0.135 & 0.039" 
# LIML -0.255 & 0.401 & 5.934 & 0.094 & -0.058 & 0.161 & 0.315 & 0.06 & -0.012 & 0.081 & 0.127 & 0.047" 
# HLIM 1.633 & 0.602 & 48.693 & 0.16 & -0.275 & 0.185 & 3.012 & 0.045 & -0.026 & 0.083 & 0.135 & 0.039" 
# HFUL 0.06 & 0.394 & 5.869 & 0.102 & -0.056 & 0.16 & 0.315 & 0.061 & -0.011 & 0.08 & 0.127 & 0.048"

#----------------------------------------------------------------------------------------->

Res.B1=round(cbind(apply(Bias1b[1:11,],1,mean),apply(Bias1b[1:11,],1,MAD),apply(Bias1b[1:11,],1,RMSE),
                   apply(Bias1b[12:22,],1,mean)),3)
colnames(Res.B1)=c("MB","MAD","RMSE","Rej")
rownames(Res.B1)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.B1

Res.B2=round(cbind(apply(Bias1b.M[1:11,],1,mean),apply(Bias1b.M[1:11,],1,MAD),apply(Bias1b.M[1:11,],1,RMSE),
                   apply(Bias1b.M[12:22,],1,mean)),3)
colnames(Res.B2)=c("MB","MAD","RMSE","Rej")
rownames(Res.B2)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.B2

Res.B3=round(cbind(apply(Bias1b.MM[1:11,],1,mean),apply(Bias1b.MM[1:11,],1,MAD),apply(Bias1b.MM[1:11,],1,RMSE),
                   apply(Bias1b.MM[12:22,],1,mean)),3)
colnames(Res.B3)=c("MB","MAD","RMSE","Rej")
rownames(Res.B3)=c("MMD","WMD","WMDF","DL","ESC6","IIV","TSLS","JIVE","LIML","HLIM","HFUL")
Res.B3

cbind(Res.B1,Res.B2,Res.B3)

#        MB   MAD    RMSE   Rej     MB   MAD   RMSE   Rej     MB   MAD  RMSE   Rej
# MMD   6.308 2.619 109.930 0.023 -0.099 0.732  9.107 0.041 -0.014 0.273 0.418 0.042
# WMD  -1.180 2.567  96.641 0.017 -0.473 0.762 12.225 0.037 -0.021 0.276 0.430 0.037
# WMDF  0.692 2.575  30.582 0.017 -0.461 0.762 10.319 0.037 -0.020 0.277 0.429 0.037
# DL   -2.964 1.709  97.165 0.027 -0.062 0.398  0.661 0.039 -0.014 0.228 0.351 0.043
# ESC6 -0.009 2.528  58.278 0.023 -0.178 0.625  1.864 0.049 -0.007 0.217 0.333 0.051
# IIV  -6.919 2.581 181.213 0.018 -0.186 0.761  5.143 0.040 -0.015 0.275 0.424 0.039
# TSLS  7.926 1.607 248.133 0.026 -0.048 0.344  0.555 0.040 -0.009 0.180 0.281 0.042
# JIVE  3.388 1.681  57.376 0.089 -0.187 0.368  1.555 0.034 -0.021 0.183 0.285 0.040
# LIML  7.926 1.607 248.133 0.026 -0.048 0.344  0.555 0.040 -0.009 0.180 0.281 0.042
# HLIM  3.388 1.681  57.376 0.089 -0.187 0.368  1.555 0.034 -0.021 0.183 0.285 0.040
# HFUL  0.038 1.699  42.708 0.026 -0.050 0.346  0.554 0.042 -0.005 0.179 0.279 0.044

#for pasting into TeX
apply(cbind(Res.B1,Res.B2,Res.B3),1,paste,collapse=" & ")

# MMD 6.308 & 2.619 & 109.93 & 0.023 & -0.099 & 0.732 & 9.107 & 0.041 & -0.014 & 0.273 & 0.418 & 0.042" 
# WMD -1.18 & 2.567 & 96.641 & 0.017 & -0.473 & 0.762 & 12.225 & 0.037 & -0.021 & 0.276 & 0.43 & 0.037" 
# WMDF 0.692 & 2.575 & 30.582 & 0.017 & -0.461 & 0.762 & 10.319 & 0.037 & -0.02 & 0.277 & 0.429 & 0.037" 
# DL -2.964 & 1.709 & 97.165 & 0.027 & -0.062 & 0.398 & 0.661 & 0.039 & -0.014 & 0.228 & 0.351 & 0.043" 
# ESC6 -0.009 & 2.528 & 58.278 & 0.023 & -0.178 & 0.625 & 1.864 & 0.049 & -0.007 & 0.217 & 0.333 & 0.051" 
# IIV -6.919 & 2.581 & 181.213 & 0.018 & -0.186 & 0.761 & 5.143 & 0.04 & -0.015 & 0.275 & 0.424 & 0.039" 
# TSLS 7.926 & 1.607 & 248.133 & 0.026 & -0.048 & 0.344 & 0.555 & 0.04 & -0.009 & 0.18 & 0.281 & 0.042" 
# JIVE 3.388 & 1.681 & 57.376 & 0.089 & -0.187 & 0.368 & 1.555 & 0.034 & -0.021 & 0.183 & 0.285 & 0.04" 
# LIML 7.926 & 1.607 & 248.133 & 0.026 & -0.048 & 0.344 & 0.555 & 0.04 & -0.009 & 0.18 & 0.281 & 0.042" 
# HLIM 3.388 & 1.681 & 57.376 & 0.089 & -0.187 & 0.368 & 1.555 & 0.034 & -0.021 & 0.183 & 0.285 & 0.04" 
# HFUL 0.038 & 1.699 & 42.708 & 0.026 & -0.05 & 0.346 & 0.554 & 0.042 & -0.005 & 0.179 & 0.279 & 0.044"

#=========================================================================================>